==> gatherer-2.99.0.0331.zip: Gatherer/Swag/LibSwag.lua <==
--[[

	LibSwag - A library to help you keep track of all your swag.

	Revision: $Id: LibSwag.lua 331 2006-12-04 21:26:34Z mentalpower $

	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit licence to use this AddOn with these facilities
		since that is it's designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat

-----------------------------------------------------------------------------------

USAGE:

  * Add LibSwagData to your addons .toc file as a SavedVariable
  * Call LibSwag.Init() when your addon is fully loaded (ie your SV's are loaded)
  * Call LibSwag.RegisterHook( <<yourFunction>> ) to recieve swag data.

yourFunction will be called with the following parameters:
yourFunction(lootType, lootTable, coinAmount, extraData)
  * lootType is a string and will be one of:
  		KILL, FISH, SKIN, MINE, HERB or OPEN
  * lootTable is an array of tables with the following format:
  		{ { link = "...", name = "...", count = ... }, ... }
  * coinAmount is the amount of coinage in the loot (in copper)
  * extraData is a table whose contents is dependant on the lootType:
  		For SKIN, MINE, HERB and OPEN: { tip = "...", time = "..." }
		For KILL: { unit = "...", level = ... }
		Everything else: nil

]]

-- Note, you should add this to your addon's SavedVariables
LibSwagData = {}

-- Check versioning to see if we need to upgrade the existing library
local create = false
local update = false
if (not LibSwag) then
	create = true
elseif (LibSwag.version < 0100) then
	update = true
end

-- If there's something for us to do, then build the library functions
if (create or update) then
	local myLib = {}
	myLib.version = 0100;

	-- Event handler functions
	function myLib.OnEvent(event, a1,a2)
		if (event == "LOOT_OPENED") then
			LibSwag.Loot()
		elseif ((event == "UNIT_SPELLCAST_SUCCEEDED") or
				(event == "UNIT_SPELLCAST_FAILED") or
				(event == "UNIT_SPELLCAST_INTERRUPTED") or
				(event == "UNIT_SPELLCAST_SENT")) then
			LibSwag.Spell(event, a1,a2)
		end
	end

	-- Spell names and types
	myLib.lootSpells = {
		["Herb Gathering"] = "HERB",
		["Cueillette"] = "HERB",
		["Kräutersammeln"] = "HERB",
		["Mining"] = "MINE",
		["Minage"] = "MINE",
		["Bergbau"] = "MINE",
		["Opening"] = "OPEN",
		["Ouverture"] = "OPEN",
		["Öffnen"] = "OPEN",
		["Skinning"] = "SKIN",
		["Dépeçage"] = "SKIN",
		["Kürschnerei"] = "SKIN",
	}

	-- Basic items (note, the list grows when we find new items)
	myLib.items = {
		["MINE"] = {
			["Copper Ore"] = 2770,
			["Tin Ore"] = 2771,
			["Iron Ore"] = 2772,
			["Silver Ore"] = 2775,
			["Gold Ore"] = 2776,
			["Mithril Ore"] = 3858,
			["Truesilver Ore"] = 7911,
			["Thorium Ore"] = 10620,
			["Dark Iron Ore"] = 11370,
		},
		["HERB"] = {
			["Silverleaf"] = 765,
			["Mageroyal"] = 785,
			["Peacebloom"] = 2447,
			["Earthroot"] = 2449,
			["Briarthorn"] = 2450,
			["Swiftthistle"] = 2452,
			["Bruiseweed"] = 2453,
			["Wild Steelbloom"] = 3355,
			["Kingsblood"] = 3356,
			["Liferoot"] = 3357,
			["Khadgar's Whisker"] = 3358,
			["Grave Moss"] = 3369,
			["Fadeleaf"] = 3818,
			["Wintersbite"] = 3819,
			["Stranglekelp"] = 3820,
			["Goldthorn"] = 3821,
			["Firebloom"] = 4625,
			["Wildvine"] = 8153,
			["Purple Lotus"] = 8831,
			["Arthas' Tears"] = 8836,
			["Sungrass"] = 8838,
			["Blindweed"] = 8839,
			["Ghost Mushroom"] = 8845,
			["Gromsblood"] = 8846,
			["Dreamfoil"] = 13463,
			["Golden Sansam"] = 13464,
			["Mountain Silversage"] = 13465,
			["Plaguebloom"] = 13466,
			["Icecap"] = 13467,
			["Black Lotus"] = 13468,
		},
	}

	-- Tracks spell events to determine loot sequences
	function myLib.Spell(event, caster, spell, rank, target)
		if (not LibSwag.tracker) then LibSwag.tracker = {} end
		--DEFAULT_CHAT_FRAME:AddMessage("Caught ev="..(event or "None").." ca="..(caster or "None").." sp="..(spell or "None").." ra="..(rank or "None").." ta="..(target or "None"))

		if (event == "UNIT_SPELLCAST_SENT") then
			LibSwag.SetTooltip()
			LibSwag.tracker.spell = spell or "Opening"
			LibSwag.tracker.start = GetTime()
			LibSwag.tracker.tooltip = LibSwag.GetLastTip()
		elseif (event == "UNIT_SPELLCAST_SUCCEEDED") then
			if (LibSwag.tracker.spell == spell) then
				LibSwag.tracker.ended = GetTime()
				local tip = "None"
				if (LibSwag.tracker and LibSwag.tracker.tooltip) then
					tip = LibSwag.tracker.tooltip.tip
				end
				DEFAULT_CHAT_FRAME:AddMessage("Found spell: "..spell.." ("..tip..")")
			end
		elseif ((event == "UNIT_SPELLCAST_INTERRUPTED") or (event == "UNIT_SPELLCAST_FAILED")) then
			-- Spell failed, cancel the tracking
			LibSwag.tracker.spell = nil
			LibSwag.tracker.tip = nil
			--DEFAULT_CHAT_FRAME:AddMessage("Cancelling failed spell");
		end
	end

	-- Records the loot that just happened and calls the hooked callbacks
	function myLib.RecordLoot(ltype, method)
		DEFAULT_CHAT_FRAME:AddMessage("Loot: "..ltype..":");
		local loot = {}
		local coin = 0
		local count = GetNumLootItems()
		for i = 1, count do
			local lIcon, lName, lQuantity, lQuality = GetLootSlotInfo(i)
			local lLink = GetLootSlotLink(i)
			if (not lLink and LootSlotIsCoin(i)) then
				--DEFAULT_CHAT_FRAME:AddMessage("Coin loot: " .. lName)
				local i,j,val
				i,j, val = string.find(lName, "(%d+) "..COPPER)
				if (i) then coin = coin + val end
				i,j, val = string.find(lName, "(%d+) "..SILVER)
				if (i) then coin = coin + (val*100) end
				i,j, val = string.find(lName, "(%d+) "..GOLD)
				if (i) then coin = coin + (val*10000) end
			else
				table.insert(loot, { link = lLink, name = lName, count = lQuantity })
				--DEFAULT_CHAT_FRAME:AddMessage("  "..lLink.."x"..lQuantity);
			end
		end
		if (coin > 0) then
			--DEFAULT_CHAT_FRAME:AddMessage("  Coinage: "..coin.." copper");
		end
		if method then for k,v in pairs(method) do
			--DEFAULT_CHAT_FRAME:AddMessage("  > "..k.." = "..v)
		end end

		--DEFAULT_CHAT_FRAME:AddMessage("Calling back")
		for i, callback in pairs(LibSwag.callbacks) do
			--DEFAULT_CHAT_FRAME:AddMessage(" - Callback ["..i.."]: "..callback.name)
			callback.hook(ltype, loot, coin, method)
		end
	end

	-- Allows dependant addons to hook into the loot sequence
	function myLib.RegisterHook(name, hook)
		--DEFAULT_CHAT_FRAME:AddMessage("Callback registered")
		table.insert(LibSwag.callbacks, { name = name, hook = hook })
		for i, callback in pairs(LibSwag.callbacks) do
			--DEFAULT_CHAT_FRAME:AddMessage(" - Callback ["..i.."]: "..callback.name)
		end
	end

	-- Processes the lootbox for data
	function myLib.Loot()
		local spell
		if (LibSwag.tracker and LibSwag.tracker.spell and
			LibSwag.tracker.ended and GetTime() - LibSwag.tracker.ended < 1) then
			local lastSpell = LibSwag.tracker.spell
			spell = LibSwag.lootSpells[lastSpell]
			if (not spell and LibSwagData.spells) then
				spell = LibSwagData.spells[lastSpell]
			end

			if (not spell) then
				local primary = GetLootSlotLink(1)
				if (primary) then
					primary = primary:match("item:(%d+)")
					if (primary) then
						if (not LibSwagData.ai) then LibSwagData.ai = {} end
						if (not LibSwagData.ai[lastSpell]) then LibSwagData.ai[lastSpell] = {} end
						for stype, count in pairs(LibSwagData.ai[lastSpell]) do
							if count > 1 then
								LibSwagData.ai[lastSpell][stype] = count - 1
							else
								LibSwagData.ai[lastSpell][stype] = nil
							end
						end
						if (not LibSwagData.items) then LibSwagData.items = {} end
						if (not LibSwagData.spells) then LibSwagData.spells = {} end
						if (LibSwagData.items[primary]) then
							local primeType = LibSwagData.items[primary]
							local count = LibSwagData.ai[lastSpell][primeType] or -1
							count = count + 2
							LibSwagData.ai[lastSpell][primeType] = count
							if (count >= 3) then
								LibSwagData.spells[lastSpell] = primeType
							end
						end
					end
				end
			end
		end
		if (IsFishingLoot()) then
			return myLib.RecordLoot("FISH")
		elseif (spell) then
			return myLib.RecordLoot(spell, LibSwag.tracker.tooltip)
		elseif (CheckInteractDistance("target", 3) and UnitIsDead("target")) then
			-- Most likely to be loot from kill
			return myLib.RecordLoot("KILL", { unit = UnitName("target"), level = UnitLevel("target") })
		end

	end

	-- Registers the library to recieve an event (if it does not already do so)
	function myLib.RegEvent(ev)
		if not LibSwagFrame["REG_"..ev] then
			LibSwagFrame:RegisterEvent(ev)
			LibSwagFrame["REG_"..ev] = true
		end
	end

	-- Adds a predefined item to your saved data
	function myLib.AddItem(cat, enName, id)
		local name = GetItemInfo(id)
		--DEFAULT_CHAT_FRAME:AddMessage("Got name for "..(id or 0)..": "..(name or "None"))
		if (not LibSwagData.items) then LibSwagData.items = {} end
		if (not LibSwagData.cats) then LibSwagData.cats = {} end
		if (not LibSwagData.cats[cat]) then LibSwagData.cats[cat] = {} end
		if (not LibSwagData.spells) then LibSwagData.spells = {} end
		if (not LibSwagData.ai) then LibSwagData.ai = {} end
		if (not name) then name = enName end
		LibSwagData.items[id] = { cat = cat, name = name }
		LibSwagData.cats[cat][name] = id
	end

	-- Call this function in the OnLoad function of your addon
	function myLib.Init()
		for cat, data in pairs(LibSwag.items) do
			if (not LibSwagData[cat]) then LibSwagData[cat] = {} end
			for name, itemid in pairs(data) do
				LibSwag.AddItem(cat, name, itemid)
			end
		end
	end

	-- Function to store the last set tooltip text
	function myLib.SetTooltip(line, newtext, r,g,b,a)
		--DEFAULT_CHAT_FRAME:AddMessage("Caught SetText("..(line or "")..", "..(newtext or "None")..")")
		if (not line) then
			local gtext
			for i=1, GameTooltip:NumLines() do
				gtext = (getglobal("GameTooltipTextLeft"..i):GetText() or "")
				local rTip = getglobal("GameTooltipTextRight"..i)
				if (rTip:IsVisible()) then
					gtext = gtext.." / "..(rTip:GetText() or "")
				end
				LibSwag.SetTooltip(i, gtext)
			end
			return
		end
		if (line > 1) then
--			if (newtext and string.find(newtext, UNIT_SKINNABLE)) then
--				DEFAULT_CHAT_FRAME:AddMessage("Discovered SKIN")
--				LibSwag.lastTipType = "SKIN"
--			end
			return
		end
		local now = GetTime()

		-- Check to see if we have just recorded this tip
		if (LibSwag.lastTipTime) then
			local delta = now - LibSwag.lastTipTime
			--DEFAULT_CHAT_FRAME:AddMessage("Delta = "..delta)
			if (delta < 0.8) then return end
		end

	    -- Get the original text from the tooltip
		local text = GameTooltipTextLeft1:GetText()
		--DEFAULT_CHAT_FRAME:AddMessage("Original text = "..(text or "None"))
		if (not text) then text = newtext end

		-- We're not interested in unit tooltips or any interface tips
		if (not UnitName("mouseover") and GetMouseFocus() == WorldFrame) then
			LibSwag.lastTipText = text
			LibSwag.lastTipTime = GetTime()
			--DEFAULT_CHAT_FRAME:AddMessage("Setting tip to "..(text or "None"))
		end
	end
	-- Getter for the above function
	function myLib.GetLastTip()
		-- Loot boxes should pop up pretty quickly.. if the tooltip was last
		-- set more than 10 seconds ago, then discard it.
		if (not LibSwag.lastTipTime or GetTime() - LibSwag.lastTipTime > 10) then
			LibSwag.lastTipTime = nil
			LibSwag.lastTipText = nil
			return nil
		end
		return {
			tip = LibSwag.lastTipText,
			time = LibSwag.lastTipTime
		}
	end

    -- Replace the existing function definitions with this versions' functions somehow
	if (create) then
		LibSwag = myLib
	elseif (update) then
		for item, _ in myLib do
			LibSwag[item] = myLib[item]
			myLib[item] = nil
		end
	end

	-- Create a version independant frame if it doesn't exist
	if (not LibSwagFrame) then
		-- Setup a event handler stub (which won't change between versions)
		LibSwagFrame = CreateFrame("Frame", "LibSwagFrame")
		function LibSwagFrame.OnEvent() LibSwag.OnEvent(event, arg1, arg2) end

		-- Hook into the SetText of GameTooltip to catch the original setting of the tooltip
		LibSwagFrame.ttLine = 0
		-- Hook GameTooltipTextLeft1:SetText()
		function LibSwagFrame.GameTooltipSetTextLeft1(this, ...)
			LibSwag.SetTooltip(1, ...)
			LibSwagFrame.OldGameTooltipSetText(this, ...)
		end
		LibSwagFrame.OldGameTooltipSetTextLeft1 = GameTooltipTextLeft1.SetText
		GameTooltipTextLeft1.SetText = LibSwagFrame.GameTooltipSetTextLeft1
		-- Hook GameTooltip:SetText() and keep track of the current line
		function LibSwagFrame.GameTooltipSetText(this, ...)
			local line = LibSwagFrame.ttLine + 1
			LibSwagFrame.ttLine = line
			LibSwag.SetTooltip(line, ...)
			LibSwagFrame.OldGameTooltipSetText(this, ...)
		end
		LibSwagFrame.OldGameTooltipSetText = GameTooltip.SetText
		GameTooltip.SetText = LibSwagFrame.GameTooltipSetText
		-- Hook GameTooltip:ClearLines to reset the line number
		function LibSwagFrame.GameTooltipClearLines(this)
			LibSwagFrame.ttLine = 0
			LibSwagFrame.OldGameTooltipClearLines(this)
		end
		LibSwagFrame.OldGameTooltipClearLines = GameTooltip.ClearLines
		GameTooltip.ClearLines = LibSwagFrame.GameTooltipClearLines

		-- Display the frame and set the event callback function
		LibSwagFrame:Show()
		LibSwagFrame:SetScript("OnEvent",  LibSwagFrame.OnEvent)
	end

	-- Initialize the callback table if it doesn't exist
	if (not LibSwag.callbacks) then
		LibSwag.callbacks = {}
	end

	-- Get the library to register for the relevant events
	LibSwag.RegEvent("LOOT_OPENED")
	LibSwag.RegEvent("UNIT_SPELLCAST_SENT")
	LibSwag.RegEvent("UNIT_SPELLCAST_SUCCEEDED")
	LibSwag.RegEvent("UNIT_SPELLCAST_INTERRUPTED")
	LibSwag.RegEvent("UNIT_SPELLCAST_FAILED")
end

==> gatherer-2.99.0.0347.zip: Gatherer/Swag/LibSwag.lua <==
--[[

	LibSwag - A library to help you keep track of all your swag.

	Revision: $Id: LibSwag.lua 342 2006-12-05 17:41:42Z norganna $

	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit licence to use this AddOn with these facilities
		since that is it's designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat

-----------------------------------------------------------------------------------

USAGE:

  * Add LibSwagData to your addons .toc file as a SavedVariable
  * Call LibSwag.Init() when your addon is fully loaded (ie your SV's are loaded)
  * Call LibSwag.RegisterHook( <<yourFunction>> ) to recieve swag data.

yourFunction will be called with the following parameters:
yourFunction(lootType, lootTable, coinAmount, extraData)
  * lootType is a string and will be one of:
  		KILL, FISH, SKIN, MINE, HERB or OPEN
  * lootTable is an array of tables with the following format:
  		{ { link = "...", name = "...", count = ... }, ... }
  * coinAmount is the amount of coinage in the loot (in copper)
  * extraData is a table whose contents is dependant on the lootType:
  		For SKIN, MINE, HERB and OPEN: { tip = "...", time = "..." }
		For KILL: { unit = "...", level = ... }
		Everything else: nil

]]

-- Note, you should add this to your addon's SavedVariables
LibSwagData = {}

-- Check versioning to see if we need to upgrade the existing library
local create = false
local update = false
if (not LibSwag) then
	create = true
elseif (LibSwag.version < 0100) then
	update = true
end

-- If there's something for us to do, then build the library functions
if (create or update) then
	local myLib = {}
	myLib.version = 0100;

	-- Event handler functions
	function myLib.OnEvent(event, a1,a2)
		if (event == "LOOT_OPENED") then
			LibSwag.Loot()
		elseif ((event == "UNIT_SPELLCAST_SUCCEEDED") or
				(event == "UNIT_SPELLCAST_FAILED") or
				(event == "UNIT_SPELLCAST_INTERRUPTED") or
				(event == "UNIT_SPELLCAST_SENT")) then
			LibSwag.Spell(event, a1,a2)
		end
	end

	-- Spell names and types
	myLib.lootSpells = {
		["Herb Gathering"] = "HERB",
		["Cueillette"] = "HERB",
		["Kräutersammeln"] = "HERB",
		["Mining"] = "MINE",
		["Minage"] = "MINE",
		["Bergbau"] = "MINE",
		["Opening"] = "OPEN",
		["Ouverture"] = "OPEN",
		["Öffnen"] = "OPEN",
		["Skinning"] = "SKIN",
		["Dépeçage"] = "SKIN",
		["Kürschnerei"] = "SKIN",
	}

	-- Basic items (note, the list grows when we find new items)
	myLib.items = {
		["MINE"] = {
			["Copper Ore"] = 2770,
			["Tin Ore"] = 2771,
			["Iron Ore"] = 2772,
			["Silver Ore"] = 2775,
			["Gold Ore"] = 2776,
			["Mithril Ore"] = 3858,
			["Truesilver Ore"] = 7911,
			["Thorium Ore"] = 10620,
			["Dark Iron Ore"] = 11370,
		},
		["HERB"] = {
			["Silverleaf"] = 765,
			["Mageroyal"] = 785,
			["Peacebloom"] = 2447,
			["Earthroot"] = 2449,
			["Briarthorn"] = 2450,
			["Swiftthistle"] = 2452,
			["Bruiseweed"] = 2453,
			["Wild Steelbloom"] = 3355,
			["Kingsblood"] = 3356,
			["Liferoot"] = 3357,
			["Khadgar's Whisker"] = 3358,
			["Grave Moss"] = 3369,
			["Fadeleaf"] = 3818,
			["Wintersbite"] = 3819,
			["Stranglekelp"] = 3820,
			["Goldthorn"] = 3821,
			["Firebloom"] = 4625,
			["Wildvine"] = 8153,
			["Purple Lotus"] = 8831,
			["Arthas' Tears"] = 8836,
			["Sungrass"] = 8838,
			["Blindweed"] = 8839,
			["Ghost Mushroom"] = 8845,
			["Gromsblood"] = 8846,
			["Dreamfoil"] = 13463,
			["Golden Sansam"] = 13464,
			["Mountain Silversage"] = 13465,
			["Plaguebloom"] = 13466,
			["Icecap"] = 13467,
			["Black Lotus"] = 13468,
		},
	}

	-- Tracks spell events to determine loot sequences
	function myLib.Spell(event, caster, spell, rank, target)
		if (not LibSwag.tracker) then LibSwag.tracker = {} end

		if (event == "UNIT_SPELLCAST_SENT") then
			LibSwag.SetTooltip()
			LibSwag.tracker.spell = spell or "Opening"
			LibSwag.tracker.start = GetTime()
			LibSwag.tracker.tooltip = LibSwag.GetLastTip()
		elseif (event == "UNIT_SPELLCAST_SUCCEEDED") then
			if (LibSwag.tracker.spell == spell) then
				LibSwag.tracker.ended = GetTime()
				local tip = "None"
				if (LibSwag.tracker and LibSwag.tracker.tooltip) then
					tip = LibSwag.tracker.tooltip.tip
				end
			end
		elseif ((event == "UNIT_SPELLCAST_INTERRUPTED") or (event == "UNIT_SPELLCAST_FAILED")) then
			-- Spell failed, cancel the tracking
			LibSwag.tracker.spell = nil
			LibSwag.tracker.tip = nil
		end
	end

	-- Records the loot that just happened and calls the hooked callbacks
	function myLib.RecordLoot(ltype, method)
		local loot = {}
		local coin = 0
		local count = GetNumLootItems()
		for i = 1, count do
			local lIcon, lName, lQuantity, lQuality = GetLootSlotInfo(i)
			local lLink = GetLootSlotLink(i)
			if (not lLink and LootSlotIsCoin(i)) then
				local i,j,val
				i,j, val = string.find(lName, "(%d+) "..COPPER)
				if (i) then coin = coin + val end
				i,j, val = string.find(lName, "(%d+) "..SILVER)
				if (i) then coin = coin + (val*100) end
				i,j, val = string.find(lName, "(%d+) "..GOLD)
				if (i) then coin = coin + (val*10000) end
			else
				table.insert(loot, { link = lLink, name = lName, count = lQuantity })
			end
		end

		for i, callback in pairs(LibSwag.callbacks) do
			callback.hook(ltype, loot, coin, method)
		end
	end

	-- Allows dependant addons to hook into the loot sequence
	function myLib.RegisterHook(name, hook)
		table.insert(LibSwag.callbacks, { name = name, hook = hook })
	end

	-- Processes the lootbox for data
	function myLib.Loot()
		local spell
		if (LibSwag.tracker and LibSwag.tracker.spell and
			LibSwag.tracker.ended and GetTime() - LibSwag.tracker.ended < 1) then
			local lastSpell = LibSwag.tracker.spell
			spell = LibSwag.lootSpells[lastSpell]
			if (not spell and LibSwagData.spells) then
				spell = LibSwagData.spells[lastSpell]
			end

			if (not spell) then
				local primary = GetLootSlotLink(1)
				if (primary) then
					primary = primary:match("item:(%d+)")
					if (primary) then
						if (not LibSwagData.ai) then LibSwagData.ai = {} end
						if (not LibSwagData.ai[lastSpell]) then LibSwagData.ai[lastSpell] = {} end
						for stype, count in pairs(LibSwagData.ai[lastSpell]) do
							if count > 1 then
								LibSwagData.ai[lastSpell][stype] = count - 1
							else
								LibSwagData.ai[lastSpell][stype] = nil
							end
						end
						if (not LibSwagData.items) then LibSwagData.items = {} end
						if (not LibSwagData.spells) then LibSwagData.spells = {} end
						if (LibSwagData.items[primary]) then
							local primeType = LibSwagData.items[primary]
							local count = LibSwagData.ai[lastSpell][primeType] or -1
							count = count + 2
							LibSwagData.ai[lastSpell][primeType] = count
							if (count >= 3) then
								LibSwagData.spells[lastSpell] = primeType
							end
						end
					end
				end
			end
		end
		if (IsFishingLoot()) then
			return myLib.RecordLoot("FISH")
		elseif (spell) then
			return myLib.RecordLoot(spell, LibSwag.tracker.tooltip)
		elseif (CheckInteractDistance("target", 3) and UnitIsDead("target")) then
			-- Most likely to be loot from kill
			return myLib.RecordLoot("KILL", { unit = UnitName("target"), level = UnitLevel("target") })
		end

	end

	-- Registers the library to recieve an event (if it does not already do so)
	function myLib.RegEvent(ev)
		if not LibSwagFrame["REG_"..ev] then
			LibSwagFrame:RegisterEvent(ev)
			LibSwagFrame["REG_"..ev] = true
		end
	end

	-- Adds a predefined item to your saved data
	function myLib.AddItem(cat, enName, id)
		local name = GetItemInfo(id)
		if (not LibSwagData.items) then LibSwagData.items = {} end
		if (not LibSwagData.cats) then LibSwagData.cats = {} end
		if (not LibSwagData.cats[cat]) then LibSwagData.cats[cat] = {} end
		if (not LibSwagData.spells) then LibSwagData.spells = {} end
		if (not LibSwagData.ai) then LibSwagData.ai = {} end
		if (not name) then name = enName end
		LibSwagData.items[id] = { cat = cat, name = name }
		LibSwagData.cats[cat][name] = id
	end

	-- Call this function in the OnLoad function of your addon
	function myLib.Init()
		for cat, data in pairs(LibSwag.items) do
			if (not LibSwagData[cat]) then LibSwagData[cat] = {} end
			for name, itemid in pairs(data) do
				LibSwag.AddItem(cat, name, itemid)
			end
		end
	end

	-- Function to store the last set tooltip text
	function myLib.SetTooltip(line, newtext, r,g,b,a)
		if (not line) then
			local gtext
			for i=1, GameTooltip:NumLines() do
				gtext = (getglobal("GameTooltipTextLeft"..i):GetText() or "")
				local rTip = getglobal("GameTooltipTextRight"..i)
				if (rTip:IsVisible()) then
					gtext = gtext.." / "..(rTip:GetText() or "")
				end
				LibSwag.SetTooltip(i, gtext)
			end
			return
		end
		if (line > 1) then return end
		local now = GetTime()

		-- Check to see if we have just recorded this tip
		if (LibSwag.lastTipTime) then
			local delta = now - LibSwag.lastTipTime
			if (delta < 0.8) then return end
		end

	    -- Get the original text from the tooltip
		local text = GameTooltipTextLeft1:GetText()
		if (not text) then text = newtext end

		-- We're not interested in unit tooltips or any interface tips
		if (not UnitName("mouseover") and GetMouseFocus() == WorldFrame) then
			LibSwag.lastTipText = text
			LibSwag.lastTipTime = GetTime()
		end
	end
	-- Getter for the above function
	function myLib.GetLastTip()
		-- Loot boxes should pop up pretty quickly.. if the tooltip was last
		-- set more than 10 seconds ago, then discard it.
		if (not LibSwag.lastTipTime or GetTime() - LibSwag.lastTipTime > 10) then
			LibSwag.lastTipTime = nil
			LibSwag.lastTipText = nil
			return nil
		end
		return {
			tip = LibSwag.lastTipText,
			time = LibSwag.lastTipTime
		}
	end

    -- Replace the existing function definitions with this versions' functions somehow
	if (create) then
		LibSwag = myLib
	elseif (update) then
		for item, _ in myLib do
			LibSwag[item] = myLib[item]
			myLib[item] = nil
		end
	end

	-- Create a version independant frame if it doesn't exist
	if (not LibSwagFrame) then
		-- Setup a event handler stub (which won't change between versions)
		LibSwagFrame = CreateFrame("Frame", "LibSwagFrame")
		function LibSwagFrame.OnEvent() LibSwag.OnEvent(event, arg1, arg2) end

		-- Hook into the SetText of GameTooltip to catch the original setting of the tooltip
		LibSwagFrame.ttLine = 0
		-- Hook GameTooltipTextLeft1:SetText()
		function LibSwagFrame.GameTooltipSetTextLeft1(this, ...)
			LibSwag.SetTooltip(1, ...)
			LibSwagFrame.OldGameTooltipSetTextLeft1(this, ...)
		end
		LibSwagFrame.OldGameTooltipSetTextLeft1 = GameTooltipTextLeft1.SetText
		GameTooltipTextLeft1.SetText = LibSwagFrame.GameTooltipSetTextLeft1
		-- Hook GameTooltip:SetText() and keep track of the current line
		function LibSwagFrame.GameTooltipSetText(this, ...)
			local line = LibSwagFrame.ttLine + 1
			LibSwagFrame.ttLine = line
			LibSwag.SetTooltip(line, ...)
			LibSwagFrame.OldGameTooltipSetText(this, ...)
		end
		LibSwagFrame.OldGameTooltipSetText = GameTooltip.SetText
		GameTooltip.SetText = LibSwagFrame.GameTooltipSetText
		-- Hook GameTooltip:ClearLines to reset the line number
		function LibSwagFrame.GameTooltipClearLines(this)
			LibSwagFrame.ttLine = 0
			LibSwagFrame.OldGameTooltipClearLines(this)
		end
		LibSwagFrame.OldGameTooltipClearLines = GameTooltip.ClearLines
		GameTooltip.ClearLines = LibSwagFrame.GameTooltipClearLines

		-- Display the frame and set the event callback function
		LibSwagFrame:Show()
		LibSwagFrame:SetScript("OnEvent",  LibSwagFrame.OnEvent)
	end

	-- Initialize the callback table if it doesn't exist
	if (not LibSwag.callbacks) then
		LibSwag.callbacks = {}
	end

	-- Get the library to register for the relevant events
	LibSwag.RegEvent("LOOT_OPENED")
	LibSwag.RegEvent("UNIT_SPELLCAST_SENT")
	LibSwag.RegEvent("UNIT_SPELLCAST_SUCCEEDED")
	LibSwag.RegEvent("UNIT_SPELLCAST_INTERRUPTED")
	LibSwag.RegEvent("UNIT_SPELLCAST_FAILED")
end
